@@ -1,26 +1,71 @@
-<TS language="es_UY" version="2.0">
+<TS language="es_UY" version="2.1">
 <context>
     <name>AddressBookPage</name>
     <message>
-        <source>Double-click to edit address or label</source>
-        <translation>Doble clic para editar etiqueta o dirección </translation>
+        <source>Right-click to edit address or label</source>
+        <translation>Clic derecho para editar dirección o etiqueta</translation>
     </message>
     <message>
         <source>Create a new address</source>
         <translation>Crear una nueva dirección </translation>
     </message>
+    <message>
+        <source>&amp;New</source>
+        <translation>Nuevo</translation>
+    </message>
     <message>
         <source>Copy the currently selected address to the system clipboard</source>
         <translation>Copia la dirección seleccionada al portapapeles del sistema</translation>
     </message>
+    <message>
+        <source>&amp;Copy</source>
+        <translation>Copiar</translation>
+    </message>
+    <message>
+        <source>C&amp;lose</source>
+        <translation>Cerrar</translation>
+    </message>
+    <message>
+        <source>&amp;Copy Address</source>
+        <translation>Copiar Dirección</translation>
+    </message>
+    <message>
+        <source>&amp;Export</source>
+        <translation>Exportar</translation>
+    </message>
     <message>
         <source>&amp;Delete</source>
         <translation>&amp;Borrar</translation>
     </message>
+    <message>
+        <source>Choose the address to send coins to</source>
+        <translation>Elige una dirección donde enviar monedas a</translation>
+    </message>
+    <message>
+        <source>Sending addresses</source>
+        <translation>Enviando direcciones</translation>
+    </message>
+    <message>
+        <source>Receiving addresses</source>
+        <translation>Recibiendo direcciones
+</translation>
+    </message>
+    <message>
+        <source>&amp;Edit</source>
+        <translation>Editar</translation>
+    </message>
+    <message>
+        <source>Export Address List</source>
+        <translation>Exportar Lista de Direcciones</translation>
+    </message>
     <message>
         <source>Comma separated file (*.csv)</source>
         <translation>Archivos separados por coma (*.csv)</translation>
     </message>
+    <message>
+        <source>Exporting Failed</source>
+        <translation>Exportación fallida</translation>
+    </message>
     </context>
 <context>
     <name>AddressTableModel</name>
@@ -75,14 +120,18 @@
         <source>Change passphrase</source>
         <translation>Cambiar contraseña</translation>
     </message>
-    <message>
-        <source>Enter the old and new passphrase to the wallet.</source>
-        <translation>Ingrese la contraseña anterior y la nueva de acceso a el monedero</translation>
-    </message>
     <message>
         <source>Confirm wallet encryption</source>
         <translation>Confirme el cifrado del monedero</translation>
     </message>
+    <message>
+        <source>Are you sure you wish to encrypt your wallet?</source>
+        <translation>Estas seguro que deseas encriptar tu billetera?</translation>
+    </message>
+    <message>
+        <source>Warning: The Caps Lock key is on!</source>
+        <translation>Atención: la tecla Mayusculas esta activa!</translation>
+    </message>
     <message>
         <source>Wallet encrypted</source>
         <translation>Monedero cifrado</translation>
@@ -112,6 +161,9 @@
         <translation>Fallo en el descifrado del monedero</translation>
     </message>
     </context>
+<context>
+    <name>BanTableModel</name>
+    </context>
 <context>
     <name>BitcoinGUI</name>
     <message>
@@ -134,18 +186,58 @@
         <source>Browse transaction history</source>
         <translation>Buscar en el historial de transacciones</translation>
     </message>
+    <message>
+        <source>E&amp;xit</source>
+        <translation>Salida</translation>
+    </message>
     <message>
         <source>Quit application</source>
         <translation>Salir de la aplicacion </translation>
     </message>
+    <message>
+        <source>Show information about Qt</source>
+        <translation>Mostrar informacioón sobre</translation>
+    </message>
     <message>
         <source>&amp;Options...</source>
         <translation>&amp;Opciones...</translation>
     </message>
+    <message>
+        <source>&amp;Backup Wallet...</source>
+        <translation>Respaldar Billetera</translation>
+    </message>
+    <message>
+        <source>&amp;Change Passphrase...</source>
+        <translation>Cambiar contraseña</translation>
+    </message>
+    <message>
+        <source>&amp;Sending addresses...</source>
+        <translation>Enviando direcciones</translation>
+    </message>
+    <message>
+        <source>&amp;Receiving addresses...</source>
+        <translation>Recibiendo direcciones</translation>
+    </message>
+    <message>
+        <source>Send coins to a Bitcoin address</source>
+        <translation>Enviar monedas a una dirección BItCoin</translation>
+    </message>
     <message>
         <source>Change the passphrase used for wallet encryption</source>
         <translation>Cambie la clave utilizada para el cifrado del monedero</translation>
     </message>
+    <message>
+        <source>Bitcoin</source>
+        <translation>Bitcoin</translation>
+    </message>
+    <message>
+        <source>Wallet</source>
+        <translation>Billetera</translation>
+    </message>
+    <message>
+        <source>&amp;Show / Hide</source>
+        <translation>Mostrar / Ocultar</translation>
+    </message>
     <message>
         <source>&amp;File</source>
         <translation>&amp;Archivo</translation>
@@ -162,9 +254,17 @@
         <source>Tabs toolbar</source>
         <translation>Barra de herramientas</translation>
     </message>
-    <message numerus="yes">
-        <source>%n active connection(s) to Bitcoin network</source>
-        <translation><numerusform>%n conexión activa a la red Bitcoin </numerusform><numerusform>%n conexiones activas a la red Bitcoin</numerusform></translation>
+    <message>
+        <source>Error</source>
+        <translation>Error</translation>
+    </message>
+    <message>
+        <source>Warning</source>
+        <translation>Alerta</translation>
+    </message>
+    <message>
+        <source>Information</source>
+        <translation>Información</translation>
     </message>
     <message>
         <source>Up to date</source>
@@ -174,6 +274,17 @@
         <source>Catching up...</source>
         <translation>Ponerse al dia...</translation>
     </message>
+    <message>
+        <source>Type: %1
+</source>
+        <translation>Tipo: %1
+</translation>
+    </message>
+    <message>
+        <source>Address: %1
+</source>
+        <translation>Dirección: %1</translation>
+    </message>
     <message>
         <source>Sent transaction</source>
         <translation>Transaccion enviada</translation>
@@ -196,10 +307,38 @@
     </context>
 <context>
     <name>CoinControlDialog</name>
+    <message>
+        <source>Quantity:</source>
+        <translation>Cantidad:</translation>
+    </message>
+    <message>
+        <source>Bytes:</source>
+        <translation>Bytes:</translation>
+    </message>
+    <message>
+        <source>Amount:</source>
+        <translation>AMonto:</translation>
+    </message>
+    <message>
+        <source>Priority:</source>
+        <translation>Prioridad:</translation>
+    </message>
+    <message>
+        <source>Change:</source>
+        <translation>Cambio:</translation>
+    </message>
     <message>
         <source>Date</source>
         <translation>Fecha</translation>
     </message>
+    <message>
+        <source>Confirmed</source>
+        <translation>Confirmado</translation>
+    </message>
+    <message>
+        <source>Priority</source>
+        <translation>Prioridad</translation>
+    </message>
     <message>
         <source>(no label)</source>
         <translation>(Sin etiqueta)</translation>
@@ -235,6 +374,10 @@
         <source>Edit sending address</source>
         <translation>Editar dirección de envío </translation>
     </message>
+    <message>
+        <source>The entered address "%1" is already in the address book.</source>
+        <translation>La dirección introducida "%1" ya está en la libreta de direcciones.</translation>
+    </message>
     <message>
         <source>Could not unlock wallet.</source>
         <translation>No se puede abrir el monedero.</translation>
@@ -252,6 +395,10 @@
     </context>
 <context>
     <name>Intro</name>
+    <message>
+        <source>Error</source>
+        <translation>Error</translation>
+    </message>
     </context>
 <context>
     <name>OpenURIDialog</name>
@@ -262,6 +409,10 @@
         <source>Options</source>
         <translation>Opciones</translation>
     </message>
+    <message>
+        <source>W&amp;allet</source>
+        <translation>Billetera</translation>
+    </message>
     </context>
 <context>
     <name>OverviewPage</name>
@@ -284,6 +435,10 @@
     </context>
 <context>
     <name>RPCConsole</name>
+    <message>
+        <source>&amp;Information</source>
+        <translation>Información</translation>
+    </message>
     </context>
 <context>
     <name>ReceiveCoinsDialog</name>
@@ -294,6 +449,10 @@
     </context>
 <context>
     <name>ReceiveRequestDialog</name>
+    <message>
+        <source>Copy &amp;Address</source>
+        <translation>Copiar Dirección</translation>
+    </message>
     <message>
         <source>Address</source>
         <translation>Direccion </translation>
@@ -324,6 +483,26 @@
         <source>Send Coins</source>
         <translation>Enviar monedas</translation>
     </message>
+    <message>
+        <source>Quantity:</source>
+        <translation>Cantidad:</translation>
+    </message>
+    <message>
+        <source>Bytes:</source>
+        <translation>Bytes:</translation>
+    </message>
+    <message>
+        <source>Amount:</source>
+        <translation>AMonto:</translation>
+    </message>
+    <message>
+        <source>Priority:</source>
+        <translation>Prioridad:</translation>
+    </message>
+    <message>
+        <source>Change:</source>
+        <translation>Cambio:</translation>
+    </message>
     <message>
         <source>Send to multiple recipients at once</source>
         <translation>Enviar a varios destinatarios a la vez</translation>
@@ -379,6 +558,10 @@
         <source>Alt+P</source>
         <translation>Alt+P</translation>
     </message>
+    <message>
+        <source>Pay To:</source>
+        <translation>Pagar A:</translation>
+    </message>
     </context>
 <context>
     <name>ShutdownWindow</name>
@@ -418,6 +601,10 @@
         <source>Date</source>
         <translation>Fecha</translation>
     </message>
+    <message>
+        <source>Transaction</source>
+        <translation>Transaccion</translation>
+    </message>
     <message>
         <source>unknown</source>
         <translation>desconocido</translation>
@@ -432,21 +619,29 @@
         <source>Date</source>
         <translation>Fecha</translation>
     </message>
-    <message>
-        <source>Address</source>
-        <translation>Direccion </translation>
-    </message>
     <message>
         <source>Open until %1</source>
         <translation>Abrir hasta %1</translation>
     </message>
+    <message>
+        <source>Label</source>
+        <translation>Etiqueta</translation>
+    </message>
     </context>
 <context>
     <name>TransactionView</name>
+    <message>
+        <source>Exporting Failed</source>
+        <translation>Exportación fallida</translation>
+    </message>
     <message>
         <source>Comma separated file (*.csv)</source>
         <translation>Archivos separados por coma (*.csv)</translation>
     </message>
+    <message>
+        <source>Confirmed</source>
+        <translation>Confirmado</translation>
+    </message>
     <message>
         <source>Date</source>
         <translation>Fecha</translation>
@@ -475,8 +670,28 @@
 </context>
 <context>
     <name>WalletView</name>
+    <message>
+        <source>&amp;Export</source>
+        <translation>Exportar</translation>
+    </message>
     </context>
 <context>
     <name>bitcoin-core</name>
-    </context>
+    <message>
+        <source>Options:</source>
+        <translation>Opciones:</translation>
+    </message>
+    <message>
+        <source>Information</source>
+        <translation>Información</translation>
+    </message>
+    <message>
+        <source>Warning</source>
+        <translation>Alerta</translation>
+    </message>
+    <message>
+        <source>Error</source>
+        <translation>Error</translation>
+    </message>
+</context>
 </TS>
\ No newline at end of file
